;  BattleZone

	processor 6502

;  HARDWARE EQUATES
VBLANK          equ     $01     ;VERTICAL BLANK?
INPTCTRL        equ     $01     ;Input control
;AUDC0           equ     $15     ;Audio Control Channel 0
;AUDC1           equ     $16     ;Audio Control Channel 1
;AUDF0           equ     $17     ;Audio Frequency Channel 0
;AUDF1           equ     $18     ;Audio Frequency Channel 1
AUDV0           equ     $19     ;Audio Volume Channel 0
AUDV1           equ     $1A     ;Audio Volume Channel 1
INPT0           equ     $08     ;Paddle Control Input 0
INPT1           equ     $09     ;Paddle Control Input 1
INPT2           equ     $0A     ;Paddle Control Input 2
INPT3           equ     $0B     ;Paddle Control Input 3
INPT4           equ     $0C     ;Player 0 Fire Button Input
INPT5           equ     $0D     ;Player 1 Fire Button Input




BACKGRND        equ     $20     ;Background Color
Z0C1            equ     $21     ;Palette 0 - Color 1
Z0C2            equ     $22     ;Palette 0 - Color 2
Z0C3            equ     $23     ;Palette 0 - Color 3
WSYNC           equ     $24     ;Wait For Sync
Z1C1            equ     $25     ;Palette 1 - Color 1
Z1C2            equ     $26     ;Palette 1 - Color 2
Z1C3            equ     $27     ;Palette 1 - Color 3
MSTAT           equ     $28     ;Maria Status
Z2C1            equ     $29     ;Palette 2 - Color 1
Z2C2            equ     $2A     ;Palette 2 - Color 2
Z2C3            equ     $2B     ;Palette 2 - Color 3
DPPH            equ     $2C     ;Display List List Pointer High
Z3C1            equ     $2D     ;Palette 3 - Color 1
Z3C2            equ     $2E     ;Palette 3 - Color 2
Z3C3            equ     $2F     ;Palette 3 - Color 3
DPPL            equ     $30     ;Display List List Pointer Low
Z4C1            equ     $31     ;Palette 4 - Color 1
Z4C2            equ     $32     ;Palette 4 - Color 2
Z4C3            equ     $33     ;Palette 4 - Color 3
CHARBASE        equ     $34     ;Character Base Address
Z5C1            equ     $35     ;Palette 5 - Color 1
Z5C2            equ     $36     ;Palette 5 - Color 2
Z5C3            equ     $37     ;Palette 5 - Color 3
OFFSET          equ     $38     ;Unused - Store zero here
Z6C1            equ     $39     ;Palette 6 - Color 1
Z6C2            equ     $3A     ;Palette 6 - Color 2
Z6C3            equ     $3B     ;Palette 6 - Color 3
CTRL            equ     $3C     ;Maria Control Register
Z7C1            equ     $3D     ;Palette 7 - Color 1
Z7C2            equ     $3E     ;Palette 7 - Color 2
Z7C3            equ     $3F     ;Palette 7 - Color 3

SWCHA           equ     $280    ;P0, P1 Joystick Directional Input - RLDURLDU
SWCHB           equ     $282    ;Console Switches
CTLSWA          equ     $281    ;I/O Control for SCHWA
CTLSWB          equ     $283    ;I/O Control for SCHWB

AUDF1           equ     $4000    
AUDC1           equ     $4001
AUDF2           equ     $4002
AUDC2           equ     $4003
AUDF3           equ     $4004
AUDC3           equ     $4005
AUDF4           equ     $4006
AUDC4           equ     $4007
AUDCTL          equ     $4008    ; Audio Control
RANDOM          equ     $400A    ; Random number (read-only)
SKCTLS          equ     $400F    ; Serial Port control

                SEG.U   RAM

; <-=========================================================================->
                ORG     $0040                ;$0040 - $00FF

;  LOADER TEMPORARY VARIABLES
LDRTEMP0        DS      1
LDRTEMP1        DS      1
LDRTEMP2        DS      1
LDRTEMP3        DS      1
LDRTEMP4        DS      1
LDRTEMP5        DS      1
LDRTEMP6        DS      1
LDRTEMP7        DS      1
OLDTEMP0        DS      1
OLDTEMP1        DS      1
OLDTEMP2        DS      1
OLDTEMP3        DS      1
OLDTEMP4        DS      1
OLDTEMP5        DS      1
OLDTEMP6        DS      1
OLDTEMP7        DS      1

;  SCRATCH DATA
TEMP0           DS      1
TEMP1           DS      1
TEMP2           DS      1
TEMP3           DS      1
TEMP4           DS      1
TEMP5           DS      1
TEMP6           DS      1
TEMP7           DS      1
TEMP8           DS      1
TEMP9           DS      1
TEMP10          DS      1
TEMP11          DS      1
TEMP12          DS      1
TEMP13          DS      1
TEMP14          DS      1
TEMP15          DS      1

;  CONSOLE VARIABLES
DBSELECT        DS      1                    ;DEBOUNCE BYTE FOR SELECT
DBRESET         DS      1                    ;DEBOUNCE BYTE FOR RESET
DBPAUSE         DS      1                    ;DEBOUNCE BYTE FOR PAUSE

;  HSC VARIABLES
HSCHERE         DS      1                    ;IS HIGH SCORE CART PLUGGED IN
HSCDLIFLG       DS      1                    ;FLAG USED TO TELL IF WE ARE USING THE ALTERNATE DLI FOR THE HSC
HSCDLITMP       DS      1                    ;DLI INDICATOR FOR HSC
HSCTIMER        DS      2                    ;2 BYTES - TIMER FOR ATTRACT MODE BEFORE SHOWING HIGH SCORE SCREEN

;  SOUND ZERO PAGE VARIABLE
SOUNDZP         DS      2                    ;2 BYTES - SOUND ADDRESSES

;  PROGRAM VARIABLES
RTLOCAL         DS      2                    ;2 BYTES - TIMER
SCRNSTAT        DS      1                    ;SCREEN STATE, OFF IF $FF
NEWRAND         DS      1                    ;CURRENT SEED FOR RAND ROUTINE
OLDRAND         DS      1                    ;OLD SEED FOR RAND ROUTINE
DLIFLG          DS      1                    ;FLAG FOR WHICH DLI WE ARE EXECUTING
SPTEMP0         DS      1                    ;SCRATCH DATA FOR PLOTTING POINT ON SCREEN
SPTEMP1         DS      1                    ;SCRATCH DATA FOR PLOTTING POINT ON SCREEN
SPTEMP2         DS      1                    ;SCRATCH DATA FOR PLOTTING POINT ON SCREEN

;  PLAYER VARIABLES
SCORE1          DS      4                    ;4 BYTES - PLAYER ONE'S SCORE
SCORE2          DS      4                    ;4 BYTES - PLAYER TWO'S SCORE
ADDSCOREL       DS      1                    ;AMOUNT TO ADD TO SCORE (LOW VALUE)
ADDSCOREH       DS      1                    ;AMOUNT TO ADD TO SCORE (HIGH VALUE)
DIFFICULTY      DS      1                    ;BEGINNING LEVEL - 0 = NOVICE, 1 = INTERMEDIATE, 2 = EXPERT
MODE            DS      1                    ;MODE OF PROGRAM - #SLCTSCRN, #ATTRMODE, #GAMEPLAY, #PLYDYING, #GAMEOVER
CURRPLYR        DS      1                    ;CURRENT PLAYER PLAYING ($00 = 1, $01 = 2)
BONUSPLYR       DS      4                    ;4 BYTES - SCORE AT WHICH NEXT BONUS PLAYER IS EARNED
BONUSPLY2       DS      4                    ;4 BYTES - SCORE AT WHICH NEXT BONUS PLAYER IS EARNED (PLAYER 2)
PLAYERS         DS      1                    ;NUMBER OF PLAYERS - ($00 = 1 PLAYER, $01 = TWO PLAYERS)
LEVEL           DS      2                    ;2 BYTES - CURRENT LEVEL #
LIVES           DS      2                    ;2 BYTES - LIVES FOR EACH PLAYER
STATUS          DS      2                    ;2 BYTES - STATUS OF PLAYER ($FF = WAITING, $00 = PLAYING, $01 = DYING)
BACKSCORE       DS      2                    ;2 BYTES - ADDITION OF A SCORE EACH FRAME IN THE BACKGROUND

;  SELECT SCREEN VARIABLES
JOYDB           DS      1                    ;JOYSTICK DEBOUNCE - USED FOR SELECT SCREEN
JBUTDB          DS      1                    ;JOYSTICK BUTTON DEBOUNCE - USED FOR SELECT SCREEN
CURROPT         DS      1                    ;CURRENT OPTION - USED FOR SELECT SCREEN

;  CONTROLLER VARIABLES
DBUTTON         DS      2                    ;2 BYTES - FIRE BUTTON DEBOUNCE (BY PLAYER)

;  WAIT VARIABLES
WAITFRAME       DS      1                    ;NEXT FRAME FOR ACTIVITY

; <-=========================================================================->
                ORG     $1800                ;$1800 - $203F

; <-=========================================================================->
                ORG     $2200                ;$2200 - $27FF

; <-=========================================================================->
                ORG     $4000                ;EXTENDED 16K RAM

;  SCREEN CHARACTER AREA
SCRNBUFR        DS      $1400                ;5120 BYTES OF RAM FOR SCREEN BUFFER
TOP_LINE        DS      $20                  ;32 BYTES - TOP OF SCANNER AREA
SCORLINE        DS      $20                  ;32 BYTES FOR SCORE LINE SCREEN BUFFER
INFOLINE        DS      $20                  ;32 BYTES FOR INFORMATION LINE SCREEN BUFFER
BOT_LINE        DS      $20                  ;32 BYTES - BOTTOM OF SCANNER AREA

;  DISPLAY LIST LIST
DLL             DS      $200                 ;512 BYTES - LIST OF DISPLAY LISTS

;  DISPLAY LIST AREA
DLAREA          DS      $D00                 ;3328 BYTES FOR DISPLAY LIST
TOPDLIST        DS      $10                  ;16 BYTES - DISPLAY LIST FOR TOP OF SCANNER
SCRDLIST        DS      $10                  ;16 BYTES - DISPLAY LIST FOR SCORE LINE
INFDLIST        DS      $10                  ;16 BYTES - DISPLAY LIST FOR INFORMATION LINE
BOTDLIST        DS      $10                  ;16 BYTES - DISPLAY LIST FOR BOTTOM OF SCANNER
NULDLST         DS      $05                  ;5 BYTES - NULL DISPLAY LIST (ALL ZEROS)

;  HSC STUFF
HSCRAM          DS      $200                 ;512 BYTES FOR HSC RAM
HSCDIF          DS      $20                  ;32 BYTES FOR HSC DIFFICULTY LINE

;  TUNE VARIABLES
TUNON           DS      4                    ;4 BYTES - WHETHER TUNE IS ACTIVE
TUNINDEX        DS      4                    ;4 BYTES - WHAT TUNE IS PLAYING
TUNPRIOR        DS      4                    ;4 BYTES - WHAT THE PRIORITY OF TUNE IS
TUNBASE         DS      4                    ;4 BYTES - BASE ADDRESS OF TUNE DATA
TUNBASE1        DS      4                    ;4 BYTES - HI BYTE OF BASE ADDRESS
FREQOFF         DS      4                    ;4 BYTES - OFFSET INTO DATA FOR FREQ'S
CTLOFF          DS      4                    ;4 BYTES - OFFSET INTO DATA FOR CTL'S
VOLOFF          DS      4                    ;4 BYTES - OFFSET INTO DATA FOR VOL'S
FREQTIME        DS      4                    ;4 BYTES - NUMBER FRAMES TILL NEXT FREQ
CTLTIME         DS      4                    ;4 BYTES - NUMBER FRAMES TILL NEXT CTL
VOLTIME         DS      4                    ;4 BYTES - NUMBER FRAMES TILL NEXT VOL
TUNNUM          DS      1                    ;WHAT TUNE YOU WANT - PARAMETER
TUNTEMP0        DS      1                    ;TEMP VALUE FOR TUNE DRIVER
TUNTEMP1        DS      1                    ;TEMP VALUE FOR TUNE DRIVER
TUNOFF          DS      1                    ;ARE TUNES TURNED OFF?

; <-=========================================================================->
;  CONSTANTS

;  CHARACTER DEFINITIONS
BLNKCHAR        EQU     $00                  ;BLANK CHARACTER
BARCHAR         EQU     $6A                  ;CHARACTER USED FOR BLACK BAR TO HIDE SCROLLING
LIFECHAR        EQU     $AA                  ;LIFE INDICATOR

;  SCREEN DEFINITIONS
LEFTSIDE        EQU     $10                  ;LEFT SIDE OF VISIBLE SCREEN
RGHTSIDE        EQU     $90                  ;RIGHT SIDE OF VISIBLE SCREEN
TOPSIDE         EQU     $00                  ;TOP SIDE OF VISIBLE SCREEN
BOTSIDE         EQU     $9C                  ;BOTTOM SIDE OF VISIBLE SCREEN
OFFSCRN         EQU     $B0                  ;AN OFF SCREEN LOCATION TO PUT INVIS.
NUMCOLS         EQU     $20                  ;NUMBER OF CHARACTERS PER LINE

;  GAME MODE DEFINITIONS
SLCTSCRN        EQU     $00                  ;TITLE SCREEN / SELECT SKILL LEVEL SCREEN
ATTRMODE        EQU     $01                  ;ATTRACT MODE (OR SHOWING HIGH SCORE SCREEN)
GAMEPLAY        EQU     $02                  ;GAME BEING PLAYED
PLYDYING        EQU     $03                  ;PLAYER IS IN DEATH SEQUENCE
GAMEOVER        EQU     $04                  ;GAME OVER SCREEN

;  DIFFICULTY OPTIONS
EASY            EQU     $00
MEDIUM          EQU     $01
HARD            EQU     $02

;  CONTROLLER OPTIONS
JOYSTICK        EQU     $00
PADDLES         EQU     $01
DRIVING         EQU     $02


;  KERNEL DEFINITIONS
STACKPTR        EQU     $FF                    ;WHERE STACK IS ON PAGE 1
GRAPHON	        EQU     $4B                    ;CTL VALUE FOR 320A MODE GRAPHICS, NORMAL DMA, BACKGROUND BORDER
GRAPHOFF        EQU     $7F                    ;CTL VALUE FOR GRAPHICS OFF

          SEG     ROM

;  CHARACTER SET ------------------------------------------------------------------------------------------------------------>

          ORG     $8000
GAMEFONT
         .include "CharMode.s"

SCREENSC
         .byte    $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
         .byte    $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
         .byte    $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
         .byte    $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
         .byte    $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
         .byte    $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
         .byte    $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
         .byte    $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
         .byte    $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
         .byte    $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
         .byte    $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
         .byte    $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
         .byte    $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
         .byte    $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
         .byte    $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
         .byte    $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF

;  BEGIN PROGRAM CODE ------------------------------------------------------------------------------------------------------>
START
          SEI
          CLD
          LDA     #$17                   ;LOCK IN MARIA MODE
          STA     INPTCTRL
          LDA     #GRAPHOFF
          STA     CTRL
          LDA     #$00
          STA     OFFSET                 ;FOR FUTURE EXPANSION
          STA     INPTCTRL               ;TO MAKE JOYSTICKS NOT FREEZE
          LDX     #STACKPTR
          TXS                            ;SET STACK POINTER

          LDA     #$60                   ;ENABLE RAM AT $4000 AND $6000
          STA     $470
          LDA     #$10
          STA     $478                   ;BANK ZERO FOR RAM
          STA     $47C

          JSR     CLEARALL               ;INITIAL CLEARING OF ALL MEMORY
          JSR     STRTINIT               ;ENTRY TO GAME
          JMP     MAINPROGRAM            ;GO TO GAME

CLEARALL
          LDA     #$00
          TAX
CALOOP1
          STA     $1800,X                ;CLEAR $1800-$1FFF
          STA     $1900,X
          STA     $1A00,X
          STA     $1B00,X
          STA     $1C00,X
          STA     $1D00,X
          STA     $1E00,X
          STA     $1F00,X
          STA     $2200,X                ;CLEAR $2200-$27FF
          STA     $2300,X
          STA     $2400,X
          STA     $2500,X
          STA     $2600,X
          STA     $2700,X
          STA     $4000,X                ;CLEAR $4000-$7FFF
          STA     $4100,X
          STA     $4200,X
          STA     $4300,X
          STA     $4400,X
          STA     $4500,X
          STA     $4600,X
          STA     $4700,X
          STA     $4800,X
          STA     $4900,X
          STA     $4A00,X
          STA     $4B00,X
          STA     $4C00,X
          STA     $4D00,X
          STA     $4E00,X
          STA     $4F00,X
          STA     $5000,X
          STA     $5100,X
          STA     $5200,X
          STA     $5300,X
          STA     $5400,X
          STA     $5500,X
          STA     $5600,X
          STA     $5700,X
          STA     $5800,X
          STA     $5900,X
          STA     $5A00,X
          STA     $5B00,X
          STA     $5C00,X
          STA     $5D00,X
          STA     $5E00,X
          STA     $5F00,X
          STA     $6000,X
          STA     $6100,X
          STA     $6200,X
          STA     $6300,X
          STA     $6400,X
          STA     $6500,X
          STA     $6600,X
          STA     $6700,X
          STA     $6800,X
          STA     $6900,X
          STA     $6A00,X
          STA     $6B00,X
          STA     $6C00,X
          STA     $6D00,X
          STA     $6E00,X
          STA     $6F00,X
          STA     $7000,X
          STA     $7100,X
          STA     $7200,X
          STA     $7300,X
          STA     $7400,X
          STA     $7500,X
          STA     $7600,X
          STA     $7700,X
          STA     $7800,X
          STA     $7900,X
          STA     $7A00,X
          STA     $7B00,X
          STA     $7C00,X
          STA     $7D00,X
          STA     $7E00,X
          STA     $7F00,X
          INX
          CPX     #$00
          BEQ     CAGOON1
          JMP     CALOOP1
CAGOON1
          LDX     #$00
CALOOP2
          STA     $2000,X                ;CLEAR $2000-$203F
          INX
          CPX     #$40
          BMI     CALOOP2
          LDX     #$40
CALOOP3
          STA     $00,X                  ;CLEAR ZERO PAGE ($40-$FF)
          INX
          CPX     #$00
          BNE     CALOOP3
          RTS
         
;  DO THE INITIALIZATION NEEDED AT THE VERY MINIMUM.
STRTINIT
          JSR     SCREENOF               ;TURN OFF DISPLAY
          LDA     #$00                   ;NULL OUT LOWER PART OF PAGE ZERO
          LDX     #$1F                   ;RANDOMIZE SEED
SIRRLOOP
          EOR     $00,X
          DEX
          BPL     SIRRLOOP
          STA     NEWRAND

          LDX     #$00                   ;GET JOYSTICK STATE SET UP
          STX     INPTCTRL
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB

          JSR     HSCEXIST               ;SEE IF HI SCORE CART IS THERE
          JSR     OURINIT                ;INIT IT OUR WAY
          RTS

;  TURN THE SCREEN OFF
SCREENOF
          JSR     WAITVBL                ;WAIT TILL VBLANK STARTED
          LDA     #$7F                   ;TURN GRAPHICS OFF
          STA     CTRL

          JSR     CLRSCRN                ;MOVE ALL STAMPS & CHARACTERS OFF SCREEN
SOEND
          STA     SCRNSTAT               ;ZERO SOME STATE
          RTS

;  TURN THE SCREEN OFF WITHOUT ZEROING THE SCREEN
SCREENNO
          JSR     WAITVBL                ;WAIT TILL VBLANK STARTED
          LDA     #$7F                   ;TURN GRAPHICS OFF
          STA     CTRL
          LDA     #$00
          BEQ     SOEND

;  TURN THE SCREEN ON
SCREENON
          LDA     SCRNSTAT               ;SEE IF SCREEN WAS EVEN OFF
          BNE     SOOUT

          JSR     WAITVBL
          LDA     #$FF
          STA     DLIFLG
          INC     SCRNSTAT               ;SAY THE SCREEN IS ON

          LDA     #GRAPHON
          STA     CTRL                   ;TURN GRAPHICS ON
SOOUT
          RTS

;  CLEAR THE PLAYFIELD AREA ONLY
CLRSCRN
          LDX     #$00                   ;CLEAR CHARACTER MAP
          TXA
CSCLOOP
          STA     SCRNBUFR+$0000,X
          STA     SCRNBUFR+$0100,X
          STA     SCRNBUFR+$0200,X
          STA     SCRNBUFR+$0300,X
          STA     SCRNBUFR+$0400,X
          STA     SCRNBUFR+$0500,X
          STA     SCRNBUFR+$0600,X
          STA     SCRNBUFR+$0700,X
          STA     SCRNBUFR+$0800,X
          STA     SCRNBUFR+$0900,X
          STA     SCRNBUFR+$0A00,X
          STA     SCRNBUFR+$0B00,X
          STA     SCRNBUFR+$0C00,X
          STA     SCRNBUFR+$0D00,X
          STA     SCRNBUFR+$0E00,X
          STA     SCRNBUFR+$0F00,X
          STA     SCRNBUFR+$1000,X
          STA     SCRNBUFR+$1100,X
          STA     SCRNBUFR+$1200,X
          STA     SCRNBUFR+$1300,X

          INX
          CPX     #$00
          BNE     CSCLOOP

          RTS                            ;ALL DONE

;  CLEAR ENTIRE SCREEN, INCLUDING SCORE AND INFORMATION LINES
CLRSCRN2
          LDX     #$00
          TXA
CS2LOOP
          STA     TOP_LINE,X
          STA     SCORLINE,X
          STA     INFOLINE,X
          STA     BOT_LINE,X
          INX
          CPX     #$20
          BMI     CS2LOOP
          JMP     CLRSCRN

;  A ROUTINE TO WAIT TILL VBLANK
;WAITVBL   BIT     MSTAT                  ;IS VBLANK STARTED YET?
;          BPL     WAITVBL
;          BIT     MSTAT                  ;IS VBLANK STILL STARTED?
;          BPL     WAITVBL
;          RTS

WAITVBL
          BIT     MSTAT                  ;IS VBLANK STARTED YET?
          BMI     WAITVBL
WAITVBL2
          BIT     MSTAT                  ;IS VBLANK STILL STARTED?
          BPL     WAITVBL2
          RTS

;  INITIALIZE STATE OUR WAY - USED ALSO AFTER HSC STUFF
OURINIT
          JSR     SCREENNO               ;TURN SCREEN OFF

          LDA     #$00
          STA     BACKGRND               ;PUT BACKGROUND TO BLACK

          JSR     LOADDLST               ;INITIALIZE DISPLAY LISTS

          LDA     #>(GAMEFONT)
          STA     CHARBASE               ;INSTALL THE CHARACTERS
          LDA     #<(DLL)
          STA     DPPL                   ;SET DPPL AND DPPH TO DLLIST
          LDA     #>(DLL)
          STA     DPPH

          JSR     SCREENON               ;TURN SCREEN DISPLAY ON   
          RTS

;  THIS ROUTINE INITIALIZES THE DISPLAY LISTS
LOADDLST
          LDA     #<(DLL)                    ;SETS UP THE DLL ADDRESS 
          STA     TEMP0
          LDA     #>(DLL)
          STA     TEMP1
          LDA     #<(SCRNBUFR)               ;SETS UP THE SCREEN BUFFER ADDRESS
          STA     TEMP2
          LDA     #>(SCRNBUFR)
          STA     TEMP3
          LDA     #<(DLAREA)                 ;SETS UP THE DISPLAY LIST ADDRESS
          STA     TEMP4
          LDA     #>(DLAREA)
          STA     TEMP5

          LDX     #$00
          LDY     #$00
LDLLOOP1
          LDA     DLLTOP,Y                   ;FIRST, COPY THE TOP DLL (NULL, AND SCANNER REGION)
          STA     (TEMP0),Y
          INY
          CPY     #DLLLNGT
          BMI     LDLLOOP1
          LDA     TEMP0
          CLC
          ADC     #DLLLNGT
          STA     TEMP0
          BCC     LDLLOOP1A
          INC     TEMP1
LDLLOOP1A
          LDA     #$00
          STA     TEMP6                      ;NUMBER OF LINES
LDLLOOP2
          LDY     #$00
          LDA     #$00                       ;1 SCAN LINE, NO INTERRUPT
          STA     (TEMP0),Y
          INY
          LDA     TEMP5                      ;HIGH VALUE OF CURRENT DLIST
          STA     (TEMP0),Y
          INY
          LDA     TEMP4                      ;LOW VALUE OF CURRENT DLIST
          STA     (TEMP0),Y
          LDA     TEMP0
          CLC
          ADC     #$03                       ;DLL ENTRY LENGTH
          STA     TEMP0
          BCC     LDLLOOP2A
          INC     TEMP1
LDLLOOP2A
          
          LDY     #$00
LDLLOOP3
          LDA     DLIST,Y                    ;FIRST COPY DLIST AS IS
          STA     (TEMP4),Y
          INY
          CPY     #DLISTLNG
          BMI     LDLLOOP3
          LDY     #$00
          LDA     TEMP2                      ;UPDATE CHARACTER MODE WITH CORRECT ADDRESS
          STA     (TEMP4),Y
          LDY     #$02
          LDA     TEMP3
          STA     (TEMP4),Y
          LDA     TEMP2
          CLC
          ADC     #NUMCOLS                   ;COLUMS ON EACH LINE
          STA     TEMP2
          BCC     LDLLOOP3A
          INC     TEMP3
LDLLOOP3A
          LDA     TEMP4
          CLC
          ADC     #DLISTLNG                  ;LENGTH OF DISPLAY LIST ENTRY
          STA     TEMP4
          BCC     LDLLOOP3B
          INC     TEMP5
LDLLOOP3B
          INC     TEMP6
          LDA     TEMP6
          CMP     #BOTSIDE                   ;DID WE FILL THE SCREEN?
          BCC     LDLLOOP2                   ;NO, KEEP GOING

          LDX     #$00
          LDY     #$00
LDLLOOP4
          LDA     DLLBOT,Y                   ;FINALLY, COPY THE BOTTOM DLL
          STA     (TEMP0),Y
          INY
          CPY     #DLLLNGB
          BMI     LDLLOOP4

          LDX     #$00
LDLLOOP5
          LDA     TDLIST,X
          STA     TOPDLIST,X
          LDA     SDLIST,X
          STA     SCRDLIST,X
          LDA     IDLIST,X
          STA     INFDLIST,X
          LDA     BDLIST,X
          STA     BOTDLIST,X
          INX
          CPX     #$08
          BMI     LDLLOOP5
          RTS
          
;  ******************************************************************************************  ;
;  MAIN LOOP  *******************************************************************************  ;
;  ******************************************************************************************  ;
MAINPROGRAM
          JSR     GETCOLORS

          JSR     CLRSCRN
          JSR     DRAWSCANNER
          JSR     DRAWMTNS
          JSR     DRAWLOGO

          ;LDA     #$1E
          ;STA     TEMP0
          ;LDA     #$42
          ;STA     TEMP1
          ;LDA     #$0B
          ;STA     TEMP2
          ;LDA     #$66
          ;STA     TEMP3
          ;LDA     #$01
          ;STA     TEMP15
          ;JSR     PLOTLINE

          LDA     #SLCTSCRN
          STA     MODE

          LDA     RTLOCAL+1
          CLC
          ADC     #$01
          STA     WAITFRAME     
MAINLOOP
          LDA     RTLOCAL+1
          CMP     WAITFRAME              ;EVERYTHING GETS PROCESSED ONLY ONCE PER FRAME
          BMI     MAINLOOP
          LDA     RTLOCAL+1
          CLC
          ADC     #$01
          STA     WAITFRAME
DOCONSOLE         
          JSR     SEEBALL                ;GET CONSOLE SWITCHES
          LDA     MODE
          CMP     #GAMEPLAY              ;PLAYING THE GAME?
          BEQ     DOPLAYERS
          ;JSR     SELECTOPT
          JMP     DOGLOBAL               ;NOPE - ON OPTIONS / TITLE SCREEN
DOPLAYERS
          LDX     CURRPLYR               ;SEE IF PLAYER 1 IS ACTIVE
          ;JSR     CHKSTATUS              ;ARE WE PLAYING YET?
          ;JSR     MOVEPDL                ;MOVE PLAYER'S PADDLE
          ;JSR     PADLBALL               ;CHECK THE BALL / PADDLE COLLISION
          ;JSR     CHECKEND               ;SEE IF THE RACK IS COMPLETED
DOGLOBAL
          ;JSR     MOVEBALL               ;MOVE THE BALL ON THE PLAYFIELD
          ;JSR     BALLCOLL               ;CHECK THE BALL / PLAYFIELD COLLISION
          ;JSR     DOANIMAT               ;DO ANY ANIMATIONS THAT MIGHT BE PENDING
DONEXT
          JMP     MAINLOOP

;  SEEBALL - CHECK TO SEE IF ANY CONSOLE BUTTONS WERE HIT
;  INPUT: NONE
;  USES: A
SEEBALL
          LDA     MODE                   ;THE SELECT SCREEN HAS IT'S OWN USE FOR THE 'SELECT' BUTTON
          CMP     #SLCTSCRN
          BEQ     SBRESET
          LDA     DBSELECT               ;SEE IF WE ARE DEBOUNCING SELECT
          BNE     SBADBSEL
          LDA     SWCHB                  ;SEE IF SELECT SWITCH HIT
          AND     #$02                   ;SEE IF BIT 1 OFF
          BNE     SBRESET
          LDA     #$01                   ;SET DEBOUNCE VAR
          STA     DBSELECT
          LDA     #SLCTSCRN              ;SELECT
          CMP     MODE
          BEQ     SBAOUT
          ;LDA     #SLCTSCRN
          ;STA     MODE
          JMP     INITSELECT
SBADBSEL
          LDA     SWCHB                  ;SEE IF IT HAS BEEN RELEASED
          AND     #$02
          EOR     #$02
          STA     DBSELECT
SBRESET
          LDA     DBRESET                ;SEE IF WE ARE DEBOUNCING RESET
          BNE     SBADBRES
          LDA     SWCHB                  ;SEE IF RESET SWITCH HIT
          AND     #$01                   ;SEE IF BIT 0 OFF
          BNE     SBPAUSE
          LDA     #$01                   ;SET DEBOUNCE VAR
          STA     DBRESET
          ;LDA     #GAMEPLAY              ;START
          ;STA     MODE
          JSR     INITGAME
          JMP     INITSCREEN
SBADBRES
          LDA     SWCHB                  ;SEE IF IT HAS BEEN RELEASED
          AND     #$01
          EOR     #$01
          STA     DBRESET
SBPAUSE
          LDA     DBPAUSE                ;SEE IF WE ARE DEBOUNCING PAUSE
          BNE     SBADBPAU
          LDA     SWCHB                  ;SEE IF PAUSE SWITCH HIT
          AND     #$08                   ;SEE IF BIT 2 OFF
          BNE     SBAOUT
          LDA     #$01                   ;SET DEBOUNCE VAR
          STA     DBPAUSE
          LDA     MODE
          CMP     #GAMEPLAY
          BNE     SBAOUT
          JMP     DOPAUSE                ;PAUSE
SBADBPAU
          LDA     SWCHB                  ;SEE IF IT HAS BEEN RELEASED
          AND     #$08
          EOR     #$08
          STA     DBPAUSE
SBAOUT
          RTS

DOPAUSE
          LDA     RTLOCAL                ;STACK CLOCK
          PHA
          LDA     RTLOCAL+1
          PHA
          TXA                            ;STORE X
          PHA
       ;   JSR     STOPTUN                ;TURN SOUNDS OFF
          LDX     RTLOCAL                ;SET UP TIMER FOR GRAPHICS OFF
          DEX
PLOOPA
          ;JSR     SEEBALL                ;SEE IF RESET OR SELECT HIT
          CPX     RTLOCAL
          BNE     PLJMP0
          JSR     SCREENNO               ;TOO LONG A WAIT, TURN GRAPHICS OFF
PLJMP0
          LDA     SWCHA                  ;SEE IF USER WANTS GRAPHICS BACK ON
          AND     #$F0                   ;IS JOYSTICK MOVED?
          CMP     #$F0
          BNE     PLGRAFON
          LDA     INPT4                  ;GET BUTTON BIT
          AND     #$80                   ;ISOLATE IT
          EOR     #$80                   ;INVERT IT
          BNE     PLGRAFON               ;IS BUTTON PUSHED?
          BEQ     PLJMP1
PLGRAFON
          JSR     SCREENON               ;TURN GRAPHICS ON IF NEEDED
          LDX     RTLOCAL                ;RESET TIMER
          DEX
PLJMP1
          LDA     DBPAUSE                ;SEE IF WE ARE DEBOUNCING PAUSE
          BEQ     PLNORM
          LDA     SWCHB                  ;SEE IF IT HAS BEEN RELEASED
          AND     #$08
          EOR     #$08
          STA     DBPAUSE                ;FALL THROUGH TO SEEBALL
          JMP     PLOOPA
PLNORM
          LDA     SWCHB                  ;SEE IF PAUSE BUTTON HIT
          AND     #$08
          BNE     PLOOPA                 ;IF NOT, TRY AGAIN
     ;     JSR     STARTTUN               ;TURN TUNES ON
          JSR     SCREENON               ;TURN GRAPHICS ON IF NEEDED
          PLA                            ;UNSTACK X
          TAX
          PLA                            ;UNSTACK CLOCK
          STA     RTLOCAL+1
          PLA
          STA     RTLOCAL
          LDA     #$01
          STA     DBPAUSE                ;AND, OF COURSE, DEBOUNCE PAUSE SWITCH
          RTS

;  INITSELECT - SHOW SELECT SCREEN
INITSELECT
          LDA     #SLCTSCRN
          STA     MODE

          RTS

;  INITGAME - SET UP GAME PARAMETERS
INITGAME
          LDA     #GAMEPLAY
          STA     MODE

          LDA     #$00
          STA     CURRPLYR
          STA     SCORE1
          STA     SCORE1+1
          STA     SCORE1+2
          STA     SCORE1+3
          STA     SCORE2
          STA     SCORE2+1
          STA     SCORE2+2
          STA     SCORE2+3
          STA     BACKSCORE
          STA     BACKSCORE+1
          LDA     #$00
          STA     LEVEL
          STA     LEVEL+1

          LDA     #$FF
          STA     STATUS
          STA     STATUS+1

          JSR     IGCLRSCL
          LDA     PLAYERS
          BEQ     IGSKIPPL2
          LDX     #$01
          LDA     #$03
          STA     LIVES,X
          JSR     WRTSCORE
          JSR     WRTLIVES
IGSKIPPL2
          LDX     #$00
          LDA     #$03
          STA     LIVES,X
          JSR     WRTSCORE
          JSR     WRTLIVES

          RTS

IGCLRSCL
          LDX     #$00
          TXA
IGCSLOOP
          STA     SCORLINE,X
          INX
          CPX     #$20
          BMI     IGCSLOOP
          RTS

;  INITSCREEN - SET UP CURRENT SCREEN
;  USES: A, X, Y, TEMP0, TEMP1, TEMP2, TEMP3, TEMP4, TEMP5
INITSCREEN
 
          RTS

;  GETCOLORS - SET UP GAME COLORS
GETCOLORS
          ;WHITE IS COLOR 1 FOR ALL
          LDA     #$0E                       ;WHITE
          STA     Z0C2

          LDA     #$00                       ;BLACK
          STA     Z7C2
          RTS

;  DRAWSCANNER - DRAWS THE SCANNER AREA ON THE SCREEN
DRAWSCANNER
          LDX     #$00
DRWSLOOP
          LDA     TOPDATA,X
          STA     TOP_LINE,X
          LDA     SCRDATA,X
          STA     SCORLINE,X
          LDA     INFDATA,X
          STA     INFOLINE,X
          LDA     BOTDATA,X
          STA     BOT_LINE,X
          INX
          CPX     #$20
          BMI     DRWSLOOP
DRWSLOOP2
          RTS

TOPDATA
         .byte    $00,$60,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$70,$71
         .byte    $72,$73,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$66,$61,$00
SCRDATA
         .byte    $00,$64,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$74,$75
         .byte    $76,$77,"S","C","O","R","E",$00,$00,$00,"0","0","0","0",$65,$00
INFDATA
         .byte    $00,$64,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$78,$79
         .byte    $7A,$7B,$5E,$5F,$5E,$5F,$00,$00,$00,$00,$00,$00,$00,$00,$65,$00
BOTDATA
         .byte    $00,$62,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$7C,$7D
         .byte    $7E,$7F,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$67,$63,$00

;  DRAWMTNS - DRAWS THE MOUNTAINS IN THE BACKGROUND
DRAWMTNS
          LDX     #$00
          STX     TEMP15
          INC     TEMP15
DMTNLOOP
          LDA     MTNDATA,X
          CMP     #$FF
          BEQ     DMTNEXIT
          STA     TEMP0                  ;X0
          INX
          LDA     MTNDATA,X
          STA     TEMP1                  ;X1
          INX
          LDA     MTNDATA,X
          STA     TEMP2                  ;X2
          INX
          LDA     MTNDATA,X
          STA     TEMP3                  ;X3
          INX
          TXA
          PHA
          JSR     PLOTLINE
          PLA
          TAX
          CPX     #$00
          BEQ     DMTNEXIT
          JMP     DMTNLOOP
DMTNEXIT
          RTS

;  DRAWLOGO - DRAWS THE 'BATTLEZONE' LOGO
DRAWLOGO
          LDX     #$00
          STX     TEMP15
          INC     TEMP15
DRLGLOOP
          LDA     LOGODATA,X
          CMP     #$FF
          BEQ     DRLGEXIT
          STA     TEMP0                  ;X0
          INX
          LDA     LOGODATA,X
          STA     TEMP1                  ;X1
          INX
          LDA     LOGODATA,X
          STA     TEMP2                  ;X2
          INX
          LDA     LOGODATA,X
          STA     TEMP3                  ;X3
          INX
          TXA
          PHA
          JSR     PLOTLINE
          PLA
          TAX
          CPX     #$00
          BEQ     DRLGEXIT
          JMP     DRLGLOOP
DRLGEXIT
          RTS

MTNDATA
         .byte    0,77,254,77
         .byte    0,71,14,77
         .byte    0,71,27,77
         .byte    63,77,84,71
         .byte    84,71,90,77,84,71,90,65,90,65,97,77,91,65,103,77
         .byte    97,71,104,65,105,65,111,72,111,72,125,65
         .byte    125,65,132,77
         .byte    125,65,138,77
         .byte    132,72,139,68
         .byte    139,68,160,77,189,77,224,71,224,71,239,71
         .byte    239,71,254,73
         .byte    $FF,$FF,$FF,$FF

LOGODATA
         .byte    30,91,43,82,43,82,72,84,72,84,65,86,65,86,68,88,68,88,55,91	;B
         .byte    55,91,30,91,46,88,50,85,50,85,56,85,56,85,49,86,49,86,51,88
         .byte    51,88,46,88

         .byte    66,91,90,82,90,82,103,91,103,91,85,89,85,89,66,91,82,87,88,85	;A
         .byte    88,85,92,87,92,87,82,87

         .byte    96,82,175,82,175,82,180,88,180,88,200,90,200,90,170,91	;T
         .byte    170,91,166,84,166,84,149,84,149,84,145,90,145,90,138,84	;T
         .byte    138,84,122,84,122,84,115,90,115,90,111,84,111,84,95,84	;L
         .byte    95,84,96,82

         .byte    191,82,217,83,217,83,204,85,204,85,216,86,216,86,209,88	;E
         .byte    209,88,227,90,227,90,200,90,200,90,191,82

         .byte    33,93,121,93,121,93,120,106,120,106,14,106,14,106,54,96	;Z
         .byte    54,96,33,93,83,93,49,102,49,102,74,106,74,106,83,93		;O
         .byte    94,96,107,96,107,96,106,102,106,102,91,102,91,102,94,96

         .byte    127,93,169,99,169,99,178,93,178,93,188,106,188,106,142,99	;N
         .byte    142,99,127,106,126,106,127,92

         .byte    178,93,210,94,210,94,194,97,194,97,209,99,209,99,199,101	;E
         .byte    199,101,222,104,222,104,188,106

         .byte    $FF,$FF,$FF,$FF

;  PLOTLINE - PLOTS A LINE BETWEEN X,Y AND X1,Y1
;  INPUT: TEMP0-X, TEMP1-Y, TEMP2-X1, TEMP3-Y1, TEMP15-$00=OFF $01=ON
;  OUTPUT: LINE PLOTTED ON SCREEN
;  USES: A, X, Y, TEMP0-15
;
;  // Uses Bresenham's line algorithm
;
;  TEMP0 = X0
;  TEMP1 = Y0
;  TEMP2 = X1
;  TEMP3 = Y1
;  TEMP4 = ABS[DELTA X]
;  TEMP5 = ABS[DELTA Y]
;  TEMP6 = SLOPE X (DIRECTION ONLY)
;  TEMP7 = SLOPE Y (DIRECTION ONLY)
;  TEMP8 = -TEMP4 (-DELTA X)
;  TEMP9 = -TEMP5/2 (-DELTA Y / 2)
;  TEMP10= (LOW)  COPY OF ERROR (TEMP12) FOR LOOP
;  TEMP11= (HIGH) COPY OF ERROR (TEMP13) FOR LOOP
;  TEMP12= (LOW)  ERROR;
;  TEMP13= (HIGH) ERROR;
;  TEMP14= N/A
;  TEMP15= LINE OFF ($00) OR LINE ON ($01)
;
;  void line(int TEMP0, int TEMP1, int TEMP2, int TEMP3) 
;  {   
;    int TEMP4  = abs(TEMP2-TEMP0), TEMP6 = TEMP0<TEMP2 ? 1 : -1;
;    int TEMP5  = abs(TEMP3-TEMP1), TEMP7 = TEMP1<TEMP3 ? 1 : -1;
;    int TEMP12 = (TEMP4>TEMP5 ? (TEMP4/2) : TEMP9, TEMP10;
;
;    for(;;)
;    {
;      setPixel(TEMP0,TEMP1);
;      if (TEMP0==TEMP2 && TEMP1==TEMP3) break;
;      TEMP10 = TEMP12;
;      if (TEMP10 > TEMP8) { TEMP12 -= TEMP5; TEMP0 += TEMP6; }
;      if (TEMP10 < TEMP5) { TEMP12 += TEMP4; TEMP1 += TEMP7; }
;    }
;  }
;
PLOTLINE
PLDELTAX
          LDA     TEMP0
          CMP     TEMP2
          BCC     PLDXPOS
PLDXNEG
          LDA     #$FF
          STA     TEMP6
          LDA     TEMP0
          SEC
          SBC     TEMP2
          STA     TEMP4
          JMP     PLDELTAY
PLDXPOS
          LDA     #$01
          STA     TEMP6
          LDA     TEMP2
          SEC
          SBC     TEMP0
          STA     TEMP4
          ;JMP     PLDELTAY
PLDELTAY
          LDA     TEMP1
          CMP     TEMP3
          BCC     PLDYPOS
PLDYNEG
          LDA     #$FF
          STA     TEMP7
          LDA     TEMP1
          SEC
          SBC     TEMP3
          STA     TEMP5
          JMP     PLNEGDXDY
PLDYPOS
          LDA     #$01
          STA     TEMP7
          LDA     TEMP3
          SEC
          SBC     TEMP1
          STA     TEMP5
          ;JMP     PLNEGDXDY
PLNEGDXDY
          LDA     TEMP4
          EOR     #$FF
          STA     TEMP8
          INC     TEMP8
          LDA     TEMP5
          LSR
          EOR     #$FF
          STA     TEMP9
          INC     TEMP9
PLERROR
          LDA     TEMP4
          CMP     TEMP5
          BEQ     PLNEGERR
          BCC     PLNEGERR
PLPOSERR
          LDA     #$00
          STA     TEMP13
          LDA     TEMP4
          LSR
          STA     TEMP12
          JMP     PLLOOP
PLNEGERR
          LDA     #$FF
          STA     TEMP13
          LDA     TEMP9
          ;LSR
          STA     TEMP12
          ;JMP    PLLOOP
PLLOOP
;          LDA     DBPAUSE                ;SEE IF WE ARE DEBOUNCING PAUSE
;          BNE     PLDBPAU
;          LDA     SWCHB                  ;SEE IF PAUSE SWITCH HIT
;          AND     #$08                   ;SEE IF BIT 2 OFF
;          BNE     PLLOOP
;          LDA     #$01                   ;SET DEBOUNCE VAR
;          STA     DBPAUSE
;          JMP     PLLOOP1
;PLDBPAU
;          LDA     SWCHB                  ;SEE IF IT HAS BEEN RELEASED
;          AND     #$08
;          EOR     #$08
;          STA     DBPAUSE
;          JMP     PLLOOP
;PLLOOP1
;          LDA     TEMP0
;          JSR     DISPLAYHEX
;          LDA     TEMP1
;          JSR     DISPLAYHEX2

          LDA     TEMP1
          CMP     #BOTSIDE                   ;IF WE ARE OUTSIDE THE BOUNDARIES OF THE SCREEN, EXIT
          BCS     PLEXIT
          JSR     SETPIXEL
          LDA     TEMP0                      ;IF X=X1 AND Y=Y1 EXIT
          CMP     TEMP2
          BNE     PLLOOPA
          LDA     TEMP1
          CMP     TEMP3
          BEQ     PLEXIT
PLLOOPA
          LDA     TEMP12
          STA     TEMP10
          LDA     TEMP13
          STA     TEMP11
          BPL     PLLOOPADOIT
          LDA     TEMP10
          CMP     TEMP8
          BEQ     PLLOOPB
          BCC     PLLOOPB
PLLOOPADOIT
          LDA     TEMP12
          SEC
          SBC     TEMP5
          STA     TEMP12
          BCS     PLLOOPADI2
          DEC     TEMP13
PLLOOPADI2
          LDA     TEMP0
          CLC
          ADC     TEMP6
          STA     TEMP0
PLLOOPB
          LDA     TEMP11
          BMI     PLLOOPBDOIT
          LDA     TEMP10
          CMP     TEMP5
          BEQ     PLNEXT
          BCS     PLNEXT
PLLOOPBDOIT
          LDA     TEMP12
          CLC
          ADC     TEMP4
          STA     TEMP12
          BCC     PLLOOPBDI2
          INC     TEMP13
PLLOOPBDI2
          LDA     TEMP1
          CLC
          ADC     TEMP7
          STA     TEMP1
PLNEXT
          JMP     PLLOOP
PLEXIT
          RTS
         
;  SETPIXEL - PLOTS A PIXEL ON THE SCREEN... X=0 TO 255, Y=0 TO 155
;  INPUT: TEMP0-X, TEMP1-Y, TEMP15-$00=OFF $01=ON
;  OUTPUT: PIXEL TURNED ON OR OFF ON SCREEN AT TEMP0,TEMP1 (DEPENDING ON TEMP15)
;  USES: A, X, Y, SPTEMP0, SPTEMP1, SPTEMP2
SETPIXEL
          LDX     TEMP1                      ;GET 'Y' POSITION FIRST
          LDA     SCRNLO,X
          STA     SPTEMP0
          LDA     SCRNHI,X
          STA     SPTEMP1
          LDA     TEMP0                      ;NOW CALCULATE 'X' POSITION
          LSR
          LSR
          LSR
          TAY                                ;THIS IS THE COLUMN
          LDA     TEMP0
          AND     #$07
          TAX
          LDA     SCPIXEL,X
          STA     SPTEMP2
SPPUT
          LDA     TEMP15
          BNE     SPPUTON
SPPUTOFF
          LDA     (SPTEMP0),Y
          EOR     SPTEMP2
          STA     (SPTEMP0),Y
          RTS
SPPUTON
          LDA     (SPTEMP0),Y
          ORA     SPTEMP2
          STA     (SPTEMP0),Y
          RTS
SCPIXEL
         .byte    $80,$40,$20,$10,$08,$04,$02,$01
SCRNHI
         .byte    >(SCRNBUFR+$0000),>(SCRNBUFR+$0020),>(SCRNBUFR+$0040),>(SCRNBUFR+$0060)
         .byte    >(SCRNBUFR+$0080),>(SCRNBUFR+$00A0),>(SCRNBUFR+$00C0),>(SCRNBUFR+$00E0)
         .byte    >(SCRNBUFR+$0100),>(SCRNBUFR+$0120),>(SCRNBUFR+$0140),>(SCRNBUFR+$0160)
         .byte    >(SCRNBUFR+$0180),>(SCRNBUFR+$01A0),>(SCRNBUFR+$01C0),>(SCRNBUFR+$01E0)
         .byte    >(SCRNBUFR+$0200),>(SCRNBUFR+$0220),>(SCRNBUFR+$0240),>(SCRNBUFR+$0260)
         .byte    >(SCRNBUFR+$0280),>(SCRNBUFR+$02A0),>(SCRNBUFR+$02C0),>(SCRNBUFR+$02E0)
         .byte    >(SCRNBUFR+$0300),>(SCRNBUFR+$0320),>(SCRNBUFR+$0340),>(SCRNBUFR+$0360)
         .byte    >(SCRNBUFR+$0380),>(SCRNBUFR+$03A0),>(SCRNBUFR+$03C0),>(SCRNBUFR+$03E0)
         .byte    >(SCRNBUFR+$0400),>(SCRNBUFR+$0420),>(SCRNBUFR+$0440),>(SCRNBUFR+$0460)
         .byte    >(SCRNBUFR+$0480),>(SCRNBUFR+$04A0),>(SCRNBUFR+$04C0),>(SCRNBUFR+$04E0)
         .byte    >(SCRNBUFR+$0500),>(SCRNBUFR+$0520),>(SCRNBUFR+$0540),>(SCRNBUFR+$0560)
         .byte    >(SCRNBUFR+$0580),>(SCRNBUFR+$05A0),>(SCRNBUFR+$05C0),>(SCRNBUFR+$05E0)
         .byte    >(SCRNBUFR+$0600),>(SCRNBUFR+$0620),>(SCRNBUFR+$0640),>(SCRNBUFR+$0660)
         .byte    >(SCRNBUFR+$0680),>(SCRNBUFR+$06A0),>(SCRNBUFR+$06C0),>(SCRNBUFR+$06E0)
         .byte    >(SCRNBUFR+$0700),>(SCRNBUFR+$0720),>(SCRNBUFR+$0740),>(SCRNBUFR+$0760)
         .byte    >(SCRNBUFR+$0780),>(SCRNBUFR+$07A0),>(SCRNBUFR+$07C0),>(SCRNBUFR+$07E0)
         .byte    >(SCRNBUFR+$0800),>(SCRNBUFR+$0820),>(SCRNBUFR+$0840),>(SCRNBUFR+$0860)
         .byte    >(SCRNBUFR+$0880),>(SCRNBUFR+$08A0),>(SCRNBUFR+$08C0),>(SCRNBUFR+$08E0)
         .byte    >(SCRNBUFR+$0900),>(SCRNBUFR+$0920),>(SCRNBUFR+$0940),>(SCRNBUFR+$0960)
         .byte    >(SCRNBUFR+$0980),>(SCRNBUFR+$09A0),>(SCRNBUFR+$09C0),>(SCRNBUFR+$09E0)
         .byte    >(SCRNBUFR+$0A00),>(SCRNBUFR+$0A20),>(SCRNBUFR+$0A40),>(SCRNBUFR+$0A60)
         .byte    >(SCRNBUFR+$0A80),>(SCRNBUFR+$0AA0),>(SCRNBUFR+$0AC0),>(SCRNBUFR+$0AE0)
         .byte    >(SCRNBUFR+$0B00),>(SCRNBUFR+$0B20),>(SCRNBUFR+$0B40),>(SCRNBUFR+$0B60)
         .byte    >(SCRNBUFR+$0B80),>(SCRNBUFR+$0BA0),>(SCRNBUFR+$0BC0),>(SCRNBUFR+$0BE0)
         .byte    >(SCRNBUFR+$0C00),>(SCRNBUFR+$0C20),>(SCRNBUFR+$0C40),>(SCRNBUFR+$0C60)
         .byte    >(SCRNBUFR+$0C80),>(SCRNBUFR+$0CA0),>(SCRNBUFR+$0CC0),>(SCRNBUFR+$0CE0)
         .byte    >(SCRNBUFR+$0D00),>(SCRNBUFR+$0D20),>(SCRNBUFR+$0D40),>(SCRNBUFR+$0D60)
         .byte    >(SCRNBUFR+$0D80),>(SCRNBUFR+$0DA0),>(SCRNBUFR+$0DC0),>(SCRNBUFR+$0DE0)
         .byte    >(SCRNBUFR+$0E00),>(SCRNBUFR+$0E20),>(SCRNBUFR+$0E40),>(SCRNBUFR+$0E60)
         .byte    >(SCRNBUFR+$0E80),>(SCRNBUFR+$0EA0),>(SCRNBUFR+$0EC0),>(SCRNBUFR+$0EE0)
         .byte    >(SCRNBUFR+$0F00),>(SCRNBUFR+$0F20),>(SCRNBUFR+$0F40),>(SCRNBUFR+$0F60)
         .byte    >(SCRNBUFR+$0F80),>(SCRNBUFR+$0FA0),>(SCRNBUFR+$0FC0),>(SCRNBUFR+$0FE0)
         .byte    >(SCRNBUFR+$1000),>(SCRNBUFR+$1020),>(SCRNBUFR+$1040),>(SCRNBUFR+$1060)
         .byte    >(SCRNBUFR+$1080),>(SCRNBUFR+$10A0),>(SCRNBUFR+$10C0),>(SCRNBUFR+$10E0)
         .byte    >(SCRNBUFR+$1100),>(SCRNBUFR+$1120),>(SCRNBUFR+$1140),>(SCRNBUFR+$1160)
         .byte    >(SCRNBUFR+$1180),>(SCRNBUFR+$11A0),>(SCRNBUFR+$11C0),>(SCRNBUFR+$11E0)
         .byte    >(SCRNBUFR+$1200),>(SCRNBUFR+$1220),>(SCRNBUFR+$1240),>(SCRNBUFR+$1260)
         .byte    >(SCRNBUFR+$1280),>(SCRNBUFR+$12A0),>(SCRNBUFR+$12C0),>(SCRNBUFR+$12E0)
         .byte    >(SCRNBUFR+$1300),>(SCRNBUFR+$1320),>(SCRNBUFR+$1340),>(SCRNBUFR+$1360)
         .byte    >(SCRNBUFR+$1380),>(SCRNBUFR+$13A0),>(SCRNBUFR+$13C0),>(SCRNBUFR+$13E0)
SCRNLO
         .byte    <(SCRNBUFR+$0000),<(SCRNBUFR+$0020),<(SCRNBUFR+$0040),<(SCRNBUFR+$0060)
         .byte    <(SCRNBUFR+$0080),<(SCRNBUFR+$00A0),<(SCRNBUFR+$00C0),<(SCRNBUFR+$00E0)
         .byte    <(SCRNBUFR+$0100),<(SCRNBUFR+$0120),<(SCRNBUFR+$0140),<(SCRNBUFR+$0160)
         .byte    <(SCRNBUFR+$0180),<(SCRNBUFR+$01A0),<(SCRNBUFR+$01C0),<(SCRNBUFR+$01E0)
         .byte    <(SCRNBUFR+$0200),<(SCRNBUFR+$0220),<(SCRNBUFR+$0240),<(SCRNBUFR+$0260)
         .byte    <(SCRNBUFR+$0280),<(SCRNBUFR+$02A0),<(SCRNBUFR+$02C0),<(SCRNBUFR+$02E0)
         .byte    <(SCRNBUFR+$0300),<(SCRNBUFR+$0320),<(SCRNBUFR+$0340),<(SCRNBUFR+$0360)
         .byte    <(SCRNBUFR+$0380),<(SCRNBUFR+$03A0),<(SCRNBUFR+$03C0),<(SCRNBUFR+$03E0)
         .byte    <(SCRNBUFR+$0400),<(SCRNBUFR+$0420),<(SCRNBUFR+$0440),<(SCRNBUFR+$0460)
         .byte    <(SCRNBUFR+$0480),<(SCRNBUFR+$04A0),<(SCRNBUFR+$04C0),<(SCRNBUFR+$04E0)
         .byte    <(SCRNBUFR+$0500),<(SCRNBUFR+$0520),<(SCRNBUFR+$0540),<(SCRNBUFR+$0560)
         .byte    <(SCRNBUFR+$0580),<(SCRNBUFR+$05A0),<(SCRNBUFR+$05C0),<(SCRNBUFR+$05E0)
         .byte    <(SCRNBUFR+$0600),<(SCRNBUFR+$0620),<(SCRNBUFR+$0640),<(SCRNBUFR+$0660)
         .byte    <(SCRNBUFR+$0680),<(SCRNBUFR+$06A0),<(SCRNBUFR+$06C0),<(SCRNBUFR+$06E0)
         .byte    <(SCRNBUFR+$0700),<(SCRNBUFR+$0720),<(SCRNBUFR+$0740),<(SCRNBUFR+$0760)
         .byte    <(SCRNBUFR+$0780),<(SCRNBUFR+$07A0),<(SCRNBUFR+$07C0),<(SCRNBUFR+$07E0)
         .byte    <(SCRNBUFR+$0800),<(SCRNBUFR+$0820),<(SCRNBUFR+$0840),<(SCRNBUFR+$0860)
         .byte    <(SCRNBUFR+$0880),<(SCRNBUFR+$08A0),<(SCRNBUFR+$08C0),<(SCRNBUFR+$08E0)
         .byte    <(SCRNBUFR+$0900),<(SCRNBUFR+$0920),<(SCRNBUFR+$0940),<(SCRNBUFR+$0960)
         .byte    <(SCRNBUFR+$0980),<(SCRNBUFR+$09A0),<(SCRNBUFR+$09C0),<(SCRNBUFR+$09E0)
         .byte    <(SCRNBUFR+$0A00),<(SCRNBUFR+$0A20),<(SCRNBUFR+$0A40),<(SCRNBUFR+$0A60)
         .byte    <(SCRNBUFR+$0A80),<(SCRNBUFR+$0AA0),<(SCRNBUFR+$0AC0),<(SCRNBUFR+$0AE0)
         .byte    <(SCRNBUFR+$0B00),<(SCRNBUFR+$0B20),<(SCRNBUFR+$0B40),<(SCRNBUFR+$0B60)
         .byte    <(SCRNBUFR+$0B80),<(SCRNBUFR+$0BA0),<(SCRNBUFR+$0BC0),<(SCRNBUFR+$0BE0)
         .byte    <(SCRNBUFR+$0C00),<(SCRNBUFR+$0C20),<(SCRNBUFR+$0C40),<(SCRNBUFR+$0C60)
         .byte    <(SCRNBUFR+$0C80),<(SCRNBUFR+$0CA0),<(SCRNBUFR+$0CC0),<(SCRNBUFR+$0CE0)
         .byte    <(SCRNBUFR+$0D00),<(SCRNBUFR+$0D20),<(SCRNBUFR+$0D40),<(SCRNBUFR+$0D60)
         .byte    <(SCRNBUFR+$0D80),<(SCRNBUFR+$0DA0),<(SCRNBUFR+$0DC0),<(SCRNBUFR+$0DE0)
         .byte    <(SCRNBUFR+$0E00),<(SCRNBUFR+$0E20),<(SCRNBUFR+$0E40),<(SCRNBUFR+$0E60)
         .byte    <(SCRNBUFR+$0E80),<(SCRNBUFR+$0EA0),<(SCRNBUFR+$0EC0),<(SCRNBUFR+$0EE0)
         .byte    <(SCRNBUFR+$0F00),<(SCRNBUFR+$0F20),<(SCRNBUFR+$0F40),<(SCRNBUFR+$0F60)
         .byte    <(SCRNBUFR+$0F80),<(SCRNBUFR+$0FA0),<(SCRNBUFR+$0FC0),<(SCRNBUFR+$0FE0)
         .byte    <(SCRNBUFR+$1000),<(SCRNBUFR+$1020),<(SCRNBUFR+$1040),<(SCRNBUFR+$1060)
         .byte    <(SCRNBUFR+$1080),<(SCRNBUFR+$10A0),<(SCRNBUFR+$10C0),<(SCRNBUFR+$10E0)
         .byte    <(SCRNBUFR+$1100),<(SCRNBUFR+$1120),<(SCRNBUFR+$1140),<(SCRNBUFR+$1160)
         .byte    <(SCRNBUFR+$1180),<(SCRNBUFR+$11A0),<(SCRNBUFR+$11C0),<(SCRNBUFR+$11E0)
         .byte    <(SCRNBUFR+$1200),<(SCRNBUFR+$1220),<(SCRNBUFR+$1240),<(SCRNBUFR+$1260)
         .byte    <(SCRNBUFR+$1280),<(SCRNBUFR+$12A0),<(SCRNBUFR+$12C0),<(SCRNBUFR+$12E0)
         .byte    <(SCRNBUFR+$1300),<(SCRNBUFR+$1320),<(SCRNBUFR+$1340),<(SCRNBUFR+$1360)
         .byte    <(SCRNBUFR+$1380),<(SCRNBUFR+$13A0),<(SCRNBUFR+$13C0),<(SCRNBUFR+$13E0)
;
;  KERNAL - MAINTAIN THE ON-SCREEN DISPLAY
DLI  
          PHA                            ;STACK REGISTERS
          TXA
          PHA
          TYA
          PHA
          CLD

          LDA     HSCDLIFLG
          BEQ     DLI2
          JMP     HSCDLI      
DLI2
          LDA     DLIFLG
          BMI     DLITOP
          BEQ     DLIMID
          JMP     DLIBOTTOM
DLITOP
          LDA     #>(GAMEFONT)
          STA     CHARBASE                ;INSTALL THE CHARACTERS
          LDA     #$00
          STA     DLIFLG                  ;THIS JUST TELLS THE NEXT DLI THAT WE ARE AT THE BOTTOM
          JMP     DLIOUT

DLIMID
          LDA     #>(SCREENSC)
          STA     CHARBASE                ;INSTALL THE CHARACTERS
          LDA     #$01
          STA     DLIFLG
          JMP     DLIOUT

;  THIS ROUTINE TAKES CARE OF THE LAST DLI ON THE SCREEN, THE 'VERTICAL BLANK
;  ROUTINE'.
DLIBOTTOM
          LDA     #$FF
          STA     DLIFLG                 ;RESET FOR TOP DLI
          INC     RTLOCAL+1              ;INCREMENT CLOCK
          BNE     DLICONT
          INC     RTLOCAL
DLICONT

DLIWAIT
          BIT     MSTAT                  ;WAIT FOR VBLANK
          BPL     DLIWAIT
DLIOTHER
          ;JSR     TUNER                  ;DO TUNES
          JSR     RAND                   ;UPDATE RANDOM NUMBER
DLIOUT
          PLA                            ;UNSTACK AND LEAVE
          TAY
          PLA                            ;THIS IS WHERE MOST DLI'S LEAVE
          TAX
          PLA
NULLRTI
          RTI

;  DISPLAY LIST INFORMATION

;  SCORE AREA DISPLAY LIST
TDLIST
         .byte    <(TOP_LINE),$60,>(TOP_LINE),$00+$00,LEFTSIDE  ;SCREEN - SCANNER TOP
         .byte    $00,$00,$00
SDLIST
         .byte    <(SCORLINE),$60,>(SCORLINE),$00+$00,LEFTSIDE  ;SCREEN - SCORE
         .byte    $00,$00,$00
IDLIST
         .byte    <(INFOLINE),$60,>(INFOLINE),$00+$00,LEFTSIDE  ;SCREEN - INFORMATION
         .byte    $00,$00,$00
BDLIST
         .byte    <(BOT_LINE),$60,>(BOT_LINE),$00+$00,LEFTSIDE  ;SCREEN - SCANNER BOTTOM
         .byte    $00,$00,$00

;  DEFAULT TEMPLATE FOR DISPLAY LIST
DLIST
         .byte    <(SCRNBUFR),$60,>(SCRNBUFR),$00+$00,LEFTSIDE  ;SCREEN - PLAYFIELD
         .byte    BARCHAR,    $40,>(GAMEFONT),$E0+$1F,LEFTSIDE  ;BLACK BAR
         .byte    BARCHAR,    $40,>(GAMEFONT),$E0+$1F,RGHTSIDE-4;BLACK BAR
         .byte    $00,$00,$00
DLISTEND

DLISTLNG  EQU     DLISTEND-DLIST

;  THIS IS THE DISPLAY LIST LIST.  THIS WILL BE DROPPED INTO RAM.
DLLTOP
;PAL
;PAL     .byte    $0F,>(NULDLST),<(NULDLST)                ;25 ADDITIONAL BLANK LINES FOR PAL
;PAL     .byte    $08,>(NULDLST),<(NULDLST)
;PAL     .byte    $0F,>(NULDLST),<(NULDLST)                ;16 BLANK LINES
;PAL     .byte    $08,>(NULDLST),<(NULDLST)                ;9 BLANK LINES
;PAL
;NTSC
;NTSC    .byte    $07,>(NULDLST),<(NULDLST)                ;8 BLANK LINES
;NTSC    .byte    $07,>(NULDLST),<(NULDLST)                ;8 BLANK LINES
;NTSC    .byte    $07,>(NULDLST),<(NULDLST)                ;8 BLANK LINES
;NTSC    .byte    $00,>(NULDLST),<(NULDLST)                ;1 BLANK LINE
;NTSC
         .byte    $A7,>(TOPDLIST),<(TOPDLIST)              ;SCANNER TOP- 8 LINES  - WITH INTERRUPT
         .byte    $27,>(SCRDLIST),<(SCRDLIST)              ;SCORE ZONE - 8 LINES
         .byte    $27,>(INFDLIST),<(INFDLIST)              ;INFO ZONE  - 8 LINES
         .byte    $27,>(BOTDLIST),<(BOTDLIST)              ;SCANNER BTM- 8 LINES
         .byte    $A3,>(NULDLST), <(NULDLST)               ;BLANK 4 LINES FOR INTERRUPT
DLLTEND

;        THERE WILL BE 156 LINES OF THE FOLLOWING (CREATED IN A LOOP):
;        .byte    $00,>(DLIST000),<(DLIST000)              ;PF ZONE 00 - 1 LINE
;                 .
;                 .
;                 .
;        .byte    $00,>(DLIST155),<(DLIST155)              ;PF ZONE 9B - 1 LINE

DLLBOT
;PAL
;PAL     .byte    $8F,>(NULDLST),<(NULDLST)                ;16 BLANK LINES - WITH INTERRUPT
;PAL     .byte    $09,>(NULDLST),<(NULDLST)                ;10 BLANK LINES
;PAL     .byte    $0F,>(NULDLST),<(NULDLST)                ;25 ADDITIONAL BLANK LINES FOR PAL
;PAL     .byte    $08,>(NULDLST),<(NULDLST)
;PAL
;NTSC
;NTSC    .byte    $87,>(NULDLST),<(NULDLST)                ;8 BLANK LINES - WITH INTERRUPT
;NTSC    .byte    $07,>(NULDLST),<(NULDLST)                ;8 BLANK LINES
;NTSC    .byte    $07,>(NULDLST),<(NULDLST)                ;8 BLANK LINES
;NTSC    .byte    $01,>(NULDLST),<(NULDLST)                ;2 BLANK LINES
;NTSC
         .byte    $00,$00,$00
DLLBEND

DLLLNGT   EQU     DLLTEND-DLLTOP
DLLLNGB   EQU     DLLBEND-DLLBOT

;  SOUND, UTILITIES, AND HIGH SCORE CART ------------------------------------------------------------------------------------>
          ORG     $D000

;---------------------------------------------------------------------------------------------------------------------------->
; SHSC.S
;         HIGH SCORE CARD INTERFACE FOR THE 7800 GAME
;
;  THIS ROUTINE CHECKS FOR THE EXISTENCE OF THE HIGH SCORE CART
HSCEXIST
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB

          LDX     #$00                   ;INITIALIZE TO ZERO
          STA     HSCHERE
          LDA     $3900                  ;SEE IF HI SCORE CART IS THERE
          CMP     #$C6
          BNE     HENOHSC
          LDA     $3904
          CMP     #$FE
          BNE     HENOHSC
          INX
          STX     HSCHERE
HENOHSC
          RTS

; HSSUCK -- SUCK IN HIGH SCORE, IF AVAILABLE.  IF NOT ZAP IT OUT.
;HSCSUCK
;          LDA     HSCHERE                ;SEE IF HI SCORE CART IS THERE
;          BEQ     SUCKOUT
;
;          LDA     #$00
;          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
;          STA     CTLSWB
;          JSR     HSETTEMP               ;SET UP TEMPLATE FOR HSC DATA BLOCK
;          JSR     HSETDIFC               ;SET DIFFICULTY
;          LDX     #<(HSCRAM)             ;AND DO IT
;          LDY     #>(HSCRAM)
;          JSR     SCREENNO
;          JSR     $3FF7                  ;DO HSCSTAT
;          JSR     SCREENON
;          LDX     #3
;SUCKHSIN
;          LDA     HSCRAM+$F,X
;          STA     HIGHSCORE,X
;          DEX
;          BPL     SUCKHSIN
;          JSR     TUNER                  ;THIS IS DONE TO MAKE UP FOR THE LOST FRAME
;          LDA     #$00
;          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
;          STA     CTLSWB
;          STA     HSCDLIFLG
;          JSR     WAITVBL
;SUCKOUT
;          RTS

;  THIS ROUTINE DISPLAYS THE HIGH SCORES FOR THE GAME
HOLDSCOR
          LDA     HSCHERE                ;SEE IF HI SCORE CART IS THERE
          BEQ     HOSOUT

          JSR     HSETTEMP               ;SET UP TEMPLATE FOR HSC DATA BLOCK
          LDA     #$04                   ;SET TIME TO SEE TABLE
          STA     HSCRAM+$0E
          JSR     HSCSETLOGO             ;TURN ON ALTERNATE DLI

          LDX     #$1F                   ;MAX BITS FOR SKILL LEVEL, SHIFTED RIGHT TWICE
HOSLOOP
          TXA                            ;SAVE X
          PHA

          JSR     HSETDIF                ;SET UP SCREEN
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB
          LDX     #<(HSCRAM)             ;AND DO IT
          LDY     #>(HSCRAM)
          JSR     $3FFA                  ;DO HSCATRCT

          PLA                            ;RESTORE X
          TAX
          DEX
          BPL     HOSLOOP
          JMP     HSCRECOVER             ;AND EXIT
HOSOUT
          RTS

;  THIS ROUTINE ENTERS A NEW HI SCORE
HNEWSCOR
          LDA     HSCHERE                ;SEE IF HI SCORE CART IS THERE
          BEQ     HOSOUT
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB

          JSR     HSETTEMP               ;SET UP TEMPLATE FOR HSC DATA BLOCK
          JSR     HSETDIFC               ;SET DIFFICULTY
          LDA     #$00
          JSR     HSETPLAY               ;SET PLAYER 1
          JSR     HSCSETLOGO             ;TURN ON ALTERNATE DLI
          LDX     #<(HSCRAM)             ;AND DO IT
          LDY     #>(HSCRAM)
          JSR     $3FFD                  ;DO HSCENTER
          LDA     PLAYERS
          BEQ     HNSSHOW

          JSR     HSCRECOVER
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB

          JSR     HSETTEMP               ;SET UP TEMPLATE FOR HSC DATA BLOCK
          JSR     HSETDIFC               ;SET DIFFICULTY
          LDA     #$01
          JSR     HSETPLAY               ;SET PLAYER 2
          JSR     HSCSETLOGO             ;TURN ON ALTERNATE DLI
          LDX     #<(HSCRAM)             ;AND DO IT
          LDY     #>(HSCRAM)
          JSR     $3FFD                  ;DO HSCENTER
HNSSHOW
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB
          LDA     #$06                   ;SET TIME TO SEE TABLE
          STA     HSCRAM+$0E             ;SHOW NEWLY ENTERED INITIALS
          LDX     #<(HSCRAM)
          LDY     #>(HSCRAM)
          JSR     $3FFA
HNSOUT
          JMP     HSCRECOVER             ;AND EXIT

;  THIS ROUTINE RECOVERS FROM THE HSC AND RETURNS TO THE ATTRACT MODE
HSCRECOVER
          JSR     SCREENNO
          ;JSR     TUNER                  ;MAKE UP FOR LOST FRAME
          LDA     #$00
          STA     CTLSWA                 ;INITIALIZE JOYSTICKS AND SWITCHES
          STA     CTLSWB
          STA     HSCDLIFLG
          JSR     OURINIT                ;FIX OUR STATE

          JMP     GETCOLORS
          RTS

;  THIS ROUTINE SETS UP A DEFAULT HSC TABLE IN THE START OF HSC RAM
HSETTEMP  LDX     #$1F
HSTLOOP   LDA     HSCTEMP,X              ;INITIALIZE HI SCORE TEMPLATE
          STA     HSCRAM,X
          LDA     #$1D                   ;CLEAR DIFFICULTY NAME
          STA     HSCDIF,X
          DEX
          BPL     HSTLOOP
          RTS

;  THIS ROUTINE SETS THE PLAYER.  A IS THE PLAYER.  HSCDIF MUST BE CALLED FIRST.
;  PLAYER 1 MUST BE DONE BEFORE PLAYER 2.
HSETPLAY  TAX
          ORA     HSCRAM+2               ;OR IN THE DIFFICULTY
          STA     HSCRAM+2

          LDA     #>(SCORE1)             ;SET HI BYTE OF SCORE
          STA     HSCRAM+9
          CPX     #$00                   ;SEE WHICH PLAYER
          BNE     HSCP2

          LDA     #<(SCORE1)             ;SET SCORE FOR PLAYER 1
          STA     HSCRAM+8
          RTS

HSCP2     LDA     #<(SCORE2)             ;SET SCORE FOR PLAYER 2
          STA     HSCRAM+8
          RTS

;  THIS ROUTINE SETS THE DIFFICULTY IN A FOR THE HSCRAM
HSETDIFC
          ;------------------------------SET UP DIFFICULTY BYTE HERE
          ;LDA     CLASSIC                ;CLASSIC MODE
          ASL                            ;SHIFT LEFT 2
          ASL
          ORA     PLAYERS                ;DIFFERENT SETTING FOR DUAL PLAYERS
          AND     #$06                   ;MAKE IT $00=SINGLE PLAYER AT A TIME, $02=DUAL PLAYERS
          ;ORA     CLRTYPE                ;$00=ROW, $01=ALL
          ASL                            ;SHIFT LEFT 2
          ASL
          ORA     DIFFICULTY             ;$00=EASY, $01=NORM, $02=HARD
          ;------------------------------
HSETDIF
          PHA                            ;SAVE THE PARAMETER IN A
          ASL                            ;SHIFT IT LEFT TWICE
          ASL
          STA     HSCRAM+2               ;STORE IT IN DIFFICULTY
          PLA
          PHA
 
          LDX     #$00                   ;FIRST BLANK OUT A 26 BYTE FIELD IN RAM (I USE 32)
          LDA     #$1D                   ;WATKINS FUNNY NOTATION OF A BLANK
HSBLANK                                  ;REPEAT
          STA     HSCDIF,X               ;  STORE A BLANK
          INX                            ;  X := X+1
          CPX     #$20
          BMI     HSBLANK                ;UNTIL (X=$20, ENTIRE FIELD BLANKED)
 
          PLA                            ;NOW WE NEED THE PARAMETER
          PHA                            ;BUT WE MUST KEEP IT FOR THE CALLER

          TAX                            ;MAKE THE DIFFICULTY LEVEL THE INDEX
          LDA     HSCINDXL,X             ;FOR THE TEXT
          STA     TEMP8
          LDA     HSCINDXH,X
          STA     TEMP9

          LDY     #$00
HSLOOP3
          LDA     (TEMP8),Y              ;GET THE NEXT CHARACTER
          BMI     HSLOOPEND              ;$FF ENDS THE LINE
HSUNDRSC
          CMP     #"_"                   ;THESE ARE SPECIAL CHARACTERS
          BNE     HSPERIOD
          LDA     #$1A+#"A"
          JMP     HSLOOPNORM
HSPERIOD
          CMP     #"."
          BNE     HSDASH
          LDA     #$1B+#"A"
          JMP     HSLOOPNORM
HSDASH
          CMP     #"-"
          BNE     HSSPACE
          LDA     #$1C+#"A"
          JMP     HSLOOPNORM
HSSPACE
          CMP     #" "                   ;A SPACE IS A DIFFERENT CHARACTER ENTIRELY
          BNE     HSBRAVE                ;REGULAR LETTER
          LDA     #$1D+#"A"              ;SPACE + "A"
          JMP     HSLOOPNORM
HSBRAVE
          CMP     #"`"
          BNE     HSBLOCK
          LDA     #$1E+#"A"
          JMP     HSLOOPNORM
HSBLOCK
          CMP     #"#"
          BNE     HSLEFT
          LDA     #$1F+#"A"
          JMP     HSLOOPNORM
HSLEFT
          CMP     #"<"
          BNE     HSQUESTION
          LDA     #$20+#"A"
          JMP     HSLOOPNORM
HSQUESTION
          CMP     #"?"
          BNE     HSNUMBER
          LDA     #$2B+#"A"
          JMP     HSLOOPNORM
HSNUMBER
          CMP     #"A"
          BPL     HSLOOPNORM
          SEC
          SBC     #$0F                   ;NUMERICAL (0-9)
          CLC
          ADC     #"A"
HSLOOPNORM
          SEC
          SBC     #"A"                   ;CONVERT TO WATKINS
          STA     HSCDIF,Y               ;STORE CHARACTER
          INY
          CPY     #$20                   ;MAXIMUM 32 POSITIONS (EVEN THOUGH ONLY 26 ARE USED)
          BMI     HSLOOP3
HSLOOPEND
          PLA                            ;RESTORE REG A
          RTS

;         THIS IS THE TEXT FOR THE NAME OF THE DIFFICULTY LEVEL
;         IT IS STORED IN WATKINS CODE, NOT ASCII (DARN!)
;         CLASSIC[6], DUALMODE[5], CLEARTYPE[4], DIFFICULTY [3..2] AND PLAYER [1..0]

HSCINDXL
         .byte    <(HSCTEXT+$000),<(HSCTEXT+$020),<(HSCTEXT+$040),<(HSCTEXT+$060)
         .byte    <(HSCTEXT+$080),<(HSCTEXT+$0A0),<(HSCTEXT+$0C0),<(HSCTEXT+$0E0)
         .byte    <(HSCTEXT+$100),<(HSCTEXT+$120),<(HSCTEXT+$140),<(HSCTEXT+$160)
         .byte    <(HSCTEXT+$180),<(HSCTEXT+$1A0),<(HSCTEXT+$1C0),<(HSCTEXT+$1E0)
         .byte    <(HSCTEXT+$200),<(HSCTEXT+$220),<(HSCTEXT+$240),<(HSCTEXT+$260)
         .byte    <(HSCTEXT+$280),<(HSCTEXT+$2A0),<(HSCTEXT+$2C0),<(HSCTEXT+$2E0)
         .byte    <(HSCTEXT+$300),<(HSCTEXT+$320),<(HSCTEXT+$340),<(HSCTEXT+$360)
         .byte    <(HSCTEXT+$380),<(HSCTEXT+$3A0),<(HSCTEXT+$3C0),<(HSCTEXT+$3E0)
HSCINDXH
         .byte    >(HSCTEXT+$000),>(HSCTEXT+$020),>(HSCTEXT+$040),>(HSCTEXT+$060)
         .byte    >(HSCTEXT+$080),>(HSCTEXT+$0A0),>(HSCTEXT+$0C0),>(HSCTEXT+$0E0)
         .byte    >(HSCTEXT+$100),>(HSCTEXT+$120),>(HSCTEXT+$140),>(HSCTEXT+$160)
         .byte    >(HSCTEXT+$180),>(HSCTEXT+$1A0),>(HSCTEXT+$1C0),>(HSCTEXT+$1E0)
         .byte    >(HSCTEXT+$200),>(HSCTEXT+$220),>(HSCTEXT+$240),>(HSCTEXT+$260)
         .byte    >(HSCTEXT+$280),>(HSCTEXT+$2A0),>(HSCTEXT+$2C0),>(HSCTEXT+$2E0)
         .byte    >(HSCTEXT+$300),>(HSCTEXT+$320),>(HSCTEXT+$340),>(HSCTEXT+$360)
         .byte    >(HSCTEXT+$380),>(HSCTEXT+$3A0),>(HSCTEXT+$3C0),>(HSCTEXT+$3E0)

HSCTEXT   ;ALLOWABLE CHARS ARE A-Z_.- `#<0-9?
         .byte    "    REFILL ROW -  EASY    ",$FF,"    ",$FF
         .byte    "    REFILL ROW -  NORM    ",$FF,"    ",$FF
         .byte    "    REFILL ROW -  HARD    ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    "    REFILL ALL -  EASY    ",$FF,"    ",$FF
         .byte    "    REFILL ALL -  NORM    ",$FF,"    ",$FF
         .byte    "    REFILL ALL -  HARD    ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    " DUAL - REFILL ROW - EASY ",$FF,"    ",$FF
         .byte    " DUAL - REFILL ROW - NORM ",$FF,"    ",$FF
         .byte    " DUAL - REFILL ROW - HARD ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    " DUAL - REFILL ALL - EASY ",$FF,"    ",$FF
         .byte    " DUAL - REFILL ALL - NORM ",$FF,"    ",$FF
         .byte    " DUAL - REFILL ALL - HARD ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ROW-EASY ",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ROW-NORM ",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ROW-HARD ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ALL-EASY ",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ALL-NORM ",$FF,"    ",$FF
         .byte    "  CLASSIC-REFILL ALL-HARD ",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ROW-EASY",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ROW-NORM",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ROW-HARD",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ALL-EASY",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ALL-NORM",$FF,"    ",$FF
         .byte    " CLASSIC-DUAL-REF ALL-HARD",$FF,"    ",$FF
         .byte    "--------------------------",$FF,"    ",$FF

;  TEMPLATE FOR HSC TABLE
HSCTEMP  .byte     $0A,$00                ;ID NUMBER AND TRAILING ZERO
         .byte     $00                    ;CLASSICMODE[6], DUALMODE[5], CLEARTYPE[4], DIFFICULTY [3..2] AND PLAYER [1..0]
         .byte     $01                    ;CONTROLLER TYPE
         .byte     $00,$00                ;POINTER TO GAME NAME (WE USE LOGO)
         .byte     <(HSCDIF),>(HSCDIF)    ;POINTER TO DIFFICULTY NAME
         .byte     $00,$00                ;POINTER TO SCORE
         .byte     <(HSCDLL),>(HSCDLL)    ;POINTER TO DLL FOR LOGO
         .byte     $00,$00;<(TUNER),>(TUNER)      ;POINTER TO SOUND ROUTINE
         .byte     $00                    ;TIME TO WAIT TILL RETURN

;<---------------------------------------------HIGH SCORE CART RPD SECTION--------------------------------------------------->
; HIGH SCORE DISPLAY LIST LIST
HSCDLL
         .byte	$8F,>(HSCNLDL),<(HSCNLDL) ;16 BLANK LINES
         .byte	$47,>(HSCNLDL),<(HSCNLDL) ;8  BLANK LINES
         .byte	$4F,>(HSCDLS0),<(HSCDLS0)
         .byte	$4F,>(HSCDLS1),<(HSCDLS1)
         .byte	$4F,>(HSCDLS2),<(HSCDLS2)
         .byte	$00,$00,$00

; HIGH SCORE DISPLAY LIST (ONLY USES CHARACTER MODE)
HSCDLS0  .byte  <(HLIST00),$60,>(HLIST00),$A0+$04,$00,$00,$00,$00
HSCDLS1  .byte  <(HLIST01),$60,>(HLIST01),$A0+$04,$00,$00,$00,$00
HSCDLS2  .byte  <(HLIST02),$60,>(HLIST02),$A0+$04,$00,$00,$00,$00
;               CHAR MODE DATA-DON'T CHNG,PAL+LNG,POS,END MARKER
HSCNLDL  .byte	$00,$00,$00

; HIGH SCORE SCREEN MEMORY - THIS IS WHERE THE CHARACTER MODE LOGO IS STORED
HLIST00  .byte    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
HLIST01  .byte    $00,$00,$00,$52,$54,$56,$58,$5A,$5C,$00,$5E,$60,$62,$64,$66,$68,$6A,$00,$00,$00
HLIST02  .byte    $00,$00,$00,$00,$00,$6C,$6E,$70,$72,$74,$76,$78,$7A,$7C,$7E,$00,$00,$00,$00,$00


; THE TOP DISPLAY LIST INTERRUPT MUST SET CHARBASE TO THE GAME CHARACTERS (AND COLORS).
; YOU MUST ALSO CREATE ANOTHER INTERRUPT TO PUT EVERYTHING BACK, BUT NOT CALL IT.
;   THE HSC DOES THAT FOR YOU. - NOTE THAT I USE 'HSCDLIFLG' FOR THIS.
;
; THE SCORE MUST BE BELOW $90 IN ZERO PAGE.  ALSO, THE HSCRAM MUST BE ON A PAGE BOUNDARY

; THIS ROUTINE SETS THE DLI FLAG ON SO THE DLI ROUTINE KNOWS TO JUMP TO HSCDLI
HSCSETLOGO
          ;JSR     TUNER                  ;MAKE UP FOR LOST FRAME
          JSR     WAITVBL
          LDA     #$01
          STA     HSCDLIFLG
          LDA     #$00
          STA     HSCDLITMP
          RTS

; THIS ROUTINE IS CALLED (INSTEAD OF THE REGULAR DLI) WHEN HSCDLIFLG IS SET TO 1
HSCDLI
          ;PHA                            ;STACK REGISTERS *THIS IS DONE IN THE STANDARD DLI*
          ;TXA
          ;PHA
          ;TYA
          ;PHA
          LDA      HSCDLITMP
          BEQ      HSCDLITOP
          JMP      HSCDLIBOT
; THIS ROUTINE IS FOR THE TOP DLI DURING HIGH SCORE CART MODE (HSCDLIFLG != 0)
HSCDLITOP
          LDA     #GRAPHON
          STA     CTRL
          LDA     #>(GAMEFONT)
          STA     CHARBASE               ;INSTALL THE GAME CHARACTER SET
          LDA     #$01                   ;SET UP FOR NORMAL DLIS
          STA     HSCDLITMP
          JMP     HSCDLIOUT

; THIS ROUTINE IS FOR THE BOTTOM DLI DURING HIGH SCORE CART MODE (HSCDLIFLG != 0).
;   IT IS NOT CALLED BY THIS PROGRAM, IT IS CALLED BY THE HSC.  IT MUST PUT BACK
;   THE HSC CHARACTER SET.
HSCDLIBOT
          LDA     #$00                   ;RESET DLI FOR TOP OF SCREEN
          STA     HSCDLITMP
          LDA     #$00
          STA     WSYNC
          STA     WSYNC
          LDA     #$39                   ;RESET HSC
          STA     CHARBASE
          LDA     #$4B
          STA     CTRL
HSCDLIOUT
          PLA
          TAY
          PLA
          TAX
          PLA
          RTI
;<---------------------------------------------HIGH SCORE CART RPD SECTION--------------------------------------------------->

;  UTILITIES ---------------------------------------------------------------------------------------------------------------->
          ORG     $F000

;  RANDOM NUMBER GENERATOR - BOTH MAKES A NEW ONE AND LEAVES IT IN A
RAND
          LDA     OLDRAND
          ADC     RTLOCAL+1              ;MAKE A PRETTY RANDOM VALUE
          ADC     NEWRAND
          PHA
          LDA     NEWRAND                ;BACK UP NEWRAND
          STA     OLDRAND
          PLA
          STA     NEWRAND                ;NEW VALUE - LEFT IN A
          RTS

;  WAIT - HALTS ALL ACTION FOR A SPECIFIED AMOUNT
;  INPUT: TIME TO WAIT IN A
;  USES: A, TEMP6, TEMP7
WAIT
          CLC
          ADC     RTLOCAL
          STA     TEMP6
          LDA     RTLOCAL+1
          STA     TEMP7
WAITLOOP
          LDA     SWCHA                  ;IF ANY KEYS ARE HIT, SKIP
          CMP     #$FF
          BNE     WEXIT
          LDA     INPT4
          AND     #$80
          BEQ     WEXIT

          LDA     RTLOCAL
          CMP     TEMP6
          BNE     WAITLOOP
          LDA     RTLOCAL+1
          CMP     TEMP7
          BNE     WAITLOOP
WEXIT
          RTS

;  SHORTWAIT - HALTS ALL ACTION FOR A SPECIFIED AMOUNT
;  INPUT: TIME TO WAIT IN A
;  USES: A, TEMP6
SHORTWAIT
          CLC
          ADC     RTLOCAL+1
          STA     TEMP6
SWLOOP
          LDA     RTLOCAL+1
          CMP     TEMP6
          BNE     SWLOOP
          RTS

;  DISPLAYHEX - FOR DEBUG ONLY - DISPLAYS THE HEX VALUE IN 'A'
;  INPUT: A
;  USES: A
DISPLAYHEX
          PHA
          LSR
          LSR
          LSR
          LSR
          CLC
          ADC      #$30
          CMP      #$3A
          BMI      DHSKIP1
          ADC      #$06
DHSKIP1
          STA      SCORLINE+$0B
          PLA
          PHA
          AND      #$0F
          CLC
          ADC      #$30
          CMP      #$3A
          BMI      DHSKIP2
          ADC      #$06
DHSKIP2
          STA      SCORLINE+$0C
          PLA
          RTS

;  DISPLAYHEX2 - FOR DEBUG ONLY - DISPLAYS THE HEX VALUE IN 'A' IN A DIFFERENT POSITION
;  INPUT: A
;  USES: A
DISPLAYHEX2
          PHA
          LSR
          LSR
          LSR
          LSR
          CLC
          ADC      #$30
          CMP      #$3A
          BMI      DH2SKIP1
          ADC      #$06
DH2SKIP1
          STA      SCORLINE+$13
          PLA
          PHA
          AND      #$0F
          CLC
          ADC      #$30
          CMP      #$3A
          BMI      DH2SKIP2
          ADC      #$06
DH2SKIP2
          STA      SCORLINE+$14
          PLA
          RTS

;  WRTSCORE - DISPLAYS THE SCORE ON THE SCREEN
;  INPUT: X = PLAYER ($00 = PLAYER 1, $01 = PLAYER 2)
;  USES: A, X, Y, (X GETS SAVED)
WRTSCORE
          TXA
          PHA
          LDA     SCORECOL,X
          TAY
          TXA
          ASL
          ASL
          TAX
          INX                            ;THE FIRST BYTE OF THE SCORE IS NOT USED (6-DIGIT ONLY)
WSBYTE1L
          LDA     SCORE1,X
          AND     #$F0                   ;LEFT DIGIT FIRST
          LSR
          LSR
          LSR
          ;LSR                           ;WE NEED TO DIVIDE BY 2 BECAUSE OF THE DOUBLE-LENGTH CHARS
          ;BEQ     WSBYTE1R               ;BLANK PADDED (NOT ZERO-FILLED)
          CLC
          ADC     #$2E
WSBYTE1R
          STA     SCORLINE,Y
          INY
          LDA     SCORE1,X
          AND     #$0F                   ;RIGHT DIGIT
          ASL
          ;BEQ     WSBYTE2L               ;BLANK PADDED (NOT ZERO-FILLED)
          CLC
          ADC     #$2E
WSBYTE2L
          STA     SCORLINE,Y
          INY
          INX
          LDA     SCORE1,X
          AND     #$F0                   ;LEFT DIGIT FIRST
          LSR
          LSR
          LSR
          ;LSR                           ;WE NEED TO DIVIDE BY 2 BECAUSE OF THE DOUBLE-LENGTH CHARS
          ;BEQ     WSBYTE2R               ;BLANK PADDED (NOT ZERO-FILLED)
          CLC
          ADC     #$2E
WSBYTE2R
          STA     SCORLINE,Y
          INY
          LDA     SCORE1,X
          AND     #$0F                   ;RIGHT DIGIT
          ASL
          ;BEQ     WSBYTE3L               ;BLANK PADDED (NOT ZERO-FILLED)
          CLC
          ADC     #$2E
WSBYTE3L
          STA     SCORLINE,Y
          INY
          INX
          LDA     SCORE1,X
          AND     #$F0                   ;LEFT DIGIT FIRST
          LSR
          LSR
          LSR
          ;LSR                           ;WE NEED TO DIVIDE BY 2 BECAUSE OF THE DOUBLE-LENGTH CHARS
          ;BEQ     WSBYTE3R               ;BLANK PADDED (NOT ZERO-FILLED)
          CLC
          ADC     #$2E
WSBYTE3R
          STA     SCORLINE,Y
          INY
          LDA     SCORE1,X
          AND     #$0F                   ;RIGHT DIGIT
          ASL
          CLC
          ADC     #$2E
          STA     SCORLINE,Y
          PLA
          TAX
          RTS

SCORECOL
         .byte    $00,$13

;  WRTLIVES - DISPLAYS THE LIVES REMAINING ON THE SCREEN
;  INPUT: X = PLAYER ($00 = PLAYER 1, $01 = PLAYER 2)
;  USES: A, X, Y, TEMP0 (X GETS SAVED)
WRTLIVES
          TXA
          PHA
          LDY     LIVESCOL,X
          LDA     LIVES,X
          STA     TEMP0
          DEC     TEMP0                    ;SHOW HOW MANY LIVES ARE *LEFT*, NOT HOW MANY THERE ARE
          LDX     #$00
WLLOOP
          CPX     TEMP0
          BMI     WLLLIFE
WLLBLNK
          LDA     #BLNKCHAR
          JMP     WLLCONT
WLLLIFE
          LDA     #LIFECHAR
WLLCONT
          STA     SCORLINE,Y
          INY
          INX
          CPX     #$03
          BMI     WLLOOP
          PLA
          TAX
          RTS

LIVESCOL
         .byte    $06,$19

;  RESERVED AREA FOR CART SIGNATURE ----------------------------------------------------------------------------------------->
          ORG     $FF80
         .byte    $FF

;  CART RESET VECTOR -------------------------------------------------------------------------------------------------------->
          ORG     $FFF8
         .byte    $FF                     ;REGION VERIFICATION
         .byte    $87                     ;ROM STARTS AT $8000
         .word    #DLI
         .word    #START
         .word    #NULLRTI
